@@ -483,7 +483,8 @@ class Marionette(object):
                         KeyError):
                     instance_class = geckoinstance.GeckoInstance
             self.instance = instance_class(host=self.host, port=self.port,
-                                           bin=self.bin, profile=self.profile, app_args=app_args)
+                                           bin=self.bin, profile=self.profile,
+                                           app_args=app_args, symbols_path=symbols_path)
             self.instance.start()
             assert(self.wait_for_port()), "Timed out waiting for port!"
 
@@ -517,7 +518,14 @@ class Marionette(object):
 
     def cleanup(self):
         if self.session:
-            self.delete_session()
+            try:
+                self.delete_session()
+            except (MarionetteException, socket.error):
+                # These exceptions get thrown if the Marionette server
+                # hit an exception/died or the connection died. We can
+                # do no further server-side cleanup in this case.
+                pass
+            self.session = None
         if self.emulator:
             self.emulator.close()
         if self.instance:
@@ -576,39 +584,39 @@ class Marionette(object):
             time.sleep(1)
         return False
 
-    def _send_message(self, command, response_key, **kwargs):
-        if not self.session and command not in ('newSession', 'getStatus'):
-            raise MarionetteException(message="Please start a session")
+    def _send_message(self, command, response_key="ok", **kwargs):
+        if not self.session and command not in ("newSession", "getStatus"):
+            raise MarionetteException("Please start a session")
 
-        message = { 'name': command }
+        message = {"name": command}
         if self.session:
-            message['sessionId'] = self.session
+            message["sessionId"] = self.session
         if kwargs:
-            message['parameters'] = kwargs
+            message["parameters"] = kwargs
 
         try:
             response = self.client.send(message)
-        except socket.timeout:
+        except socket.timeout as e:
             self.session = None
             self.window = None
             self.client.close()
-            raise TimeoutException(message='socket.timeout', status=ErrorCodes.TIMEOUT, stacktrace=None)
+            raise TimeoutException(
+                "Connection timed out", status=ErrorCodes.TIMEOUT)
 
         # Process any emulator commands that are sent from a script
         # while it's executing.
         while response.get("emulator_cmd"):
             response = self._handle_emulator_cmd(response)
 
-        if (response_key == 'ok' and response.get('ok') ==  True) or response_key in response:
+        if response_key in response:
             return response[response_key]
-        else:
-            self._handle_error(response)
+        self._handle_error(response)
 
     def _handle_emulator_cmd(self, response):
         cmd = response.get("emulator_cmd")
         if not cmd or not self.emulator:
-            raise MarionetteException(message="No emulator in this test to run "
-                                      "command against.")
+            raise MarionetteException(
+                "No emulator in this test to run command against")
         cmd = cmd.encode("ascii")
         result = self.emulator._run_telnet(cmd)
         return self.client.send({"name": "emulatorCmdResult",
@@ -679,9 +687,8 @@ class Marionette(object):
             if self.emulator.check_for_minidumps():
                 crashed = True
         elif self.instance:
-            # In the future, a check for crashed Firefox processes
-            # should be here.
-            pass
+            if self.instance.check_for_crashes():
+                crashed = True
         if returncode is not None:
             print ('PROCESS-CRASH | %s | abnormal termination with exit code %d' %
                 (name, returncode))
@@ -699,11 +706,17 @@ class Marionette(object):
         return self._send_message('getStatus', 'value')
 
     def start_session(self, desired_capabilities=None):
-        '''
-        Creates a new Marionette session.
+        """Create a new Marionette session.
+
+        This method must be called before performing any other action.
+
+        :params desired_capabilities: An optional dict of desired
+            capabilities.  This is currently ignored.
+
+        :returns: A dict of the capabilities offered.
+
+        """
 
-        You must call this method before performing any other action.
-        '''
         try:
             # We are ignoring desired_capabilities, at least for now.
             self.session = self._send_message('newSession', 'value')
