@@ -289,7 +289,7 @@ permissions.forEach(function (perm) {
                     self.loglines = [['Error getting log: %s' % inst]]
                 try:
                     self.marionette.delete_session()
-                except (socket.error, MarionetteException, IOError):
+                except (socket.error, MarionetteException):
                     # Gecko has crashed?
                     self.marionette.session = None
                     try:
@@ -419,7 +419,14 @@ class MarionetteJSTestCase(CommonTestCase):
 
         if self.oop:
             print 'running oop'
-            result = self.marionette.execute_async_script("""
+            frame = None
+            try:
+                frame = self.marionette.find_element(
+                    'css selector',
+                    'iframe[src*="app://test-container.gaiamobile.org/index.html"]'
+                )
+            except NoSuchElementException:
+                result = self.marionette.execute_async_script("""
 let setReq = navigator.mozSettings.createLock().set({'lockscreen.enabled': false});
 setReq.onsuccess = function() {
     let appsReq = navigator.mozApps.mgmt.getAll();
@@ -445,14 +452,14 @@ setReq.onsuccess = function() {
 setReq.onerror = function() {
     marionetteScriptFinished(false);
 }""", script_timeout=60000)
-            self.assertTrue(result)
+                self.assertTrue(result)
 
-            self.marionette.switch_to_frame(
-                self.marionette.find_element(
+                frame = self.marionette.find_element(
                     'css selector',
                     'iframe[src*="app://test-container.gaiamobile.org/index.html"]'
-                ))
+                )
 
+            self.marionette.switch_to_frame(frame)
             main_process = self.marionette.execute_script("""
                 return SpecialPowers.isMainProcess();
                 """)
@@ -509,5 +516,8 @@ setReq.onerror = function() {
                 self.loglines = self.marionette.get_logs()
                 raise
 
+        if self.oop:
+            self.marionette.switch_to_frame()
+
         self.marionette.execute_script("log('TEST-END: %s');" % self.jsFile.replace('\\', '\\\\'))
         self.marionette.test_name = None
