@@ -20,6 +20,7 @@ from manifestparser import TestManifest
 from mozhttpd import MozHttpd
 from marionette import Marionette
 from moztest.results import TestResultCollection, TestResult, relevant_line
+from mixins.b2g import B2GTestResultMixin, get_b2g_pid, get_dm
 
 
 class MarionetteTest(TestResult):
@@ -113,7 +114,7 @@ class MarionetteTestResult(unittest._TextTestResult, TestResultCollection):
                        context=context, **kwargs)
         # call any registered result modifiers
         for modifier in self.result_modifiers:
-            modifier(t, result_expected, result_actual, output, context)
+            result_expected, result_actual, output, context = modifier(t, result_expected, result_actual, output, context)
         t.finish(result_actual,
                  time_end=time.time() if test.start_time else 0,
                  reason=relevant_line(output),
@@ -238,6 +239,7 @@ class MarionetteTextTestRunner(unittest.TextTestRunner):
 
     def __init__(self, **kwargs):
         self.marionette = kwargs['marionette']
+        self.capabilities = kwargs.pop('capabilities')
         del kwargs['marionette']
         unittest.TextTestRunner.__init__(self, **kwargs)
 
@@ -309,6 +311,40 @@ class MarionetteTextTestRunner(unittest.TextTestRunner):
         return result
 
 
+class B2GMarionetteTestResult(MarionetteTestResult, B2GTestResultMixin):
+
+    def __init__(self, *args, **kwargs):
+        # stupid hack because _TextTestRunner doesn't accept **kwargs
+        b2g_pid = kwargs.pop('b2g_pid')
+        MarionetteTestResult.__init__(self, *args, **kwargs)
+        kwargs['b2g_pid'] = b2g_pid
+        B2GTestResultMixin.__init__(self, *args, **kwargs)
+ 
+
+class B2GMarionetteTextTestRunner(MarionetteTextTestRunner):
+
+    resultclass = B2GMarionetteTestResult
+
+    def __init__(self, **kwargs):
+        MarionetteTextTestRunner.__init__(self, **kwargs)
+        if self.capabilities['device'] != 'desktop':
+            self.resultclass = B2GMarionetteTestResult
+        self.b2g_pid = None
+
+    def _makeResult(self):
+        return self.resultclass(self.stream,
+                                self.descriptions,
+                                self.verbosity,
+                                marionette=self.marionette,
+                                b2g_pid=self.b2g_pid)
+
+    def run(self, test):
+        dm_type = os.environ.get('DM_TRANS', 'adb')
+        if dm_type == 'adb':
+            self.b2g_pid = get_b2g_pid(get_dm(self.marionette))
+        return super(B2GMarionetteTextTestRunner, self).run(test)
+
+
 class BaseMarionetteOptions(OptionParser):
     def __init__(self, **kwargs):
         OptionParser.__init__(self, **kwargs)
@@ -532,6 +568,8 @@ class BaseMarionetteTestRunner(object):
         self.shuffle = shuffle
         self.sdcard = sdcard
         self.mixin_run_tests = []
+        self.manifest_skipped_tests = []
+        self.tests = []
 
         if testvars:
             if not os.path.exists(testvars):
@@ -713,15 +751,31 @@ class BaseMarionetteTestRunner(object):
     def run_tests(self, tests):
         self.reset_test_stats()
         starttime = datetime.utcnow()
+
+        if not self.httpd:
+            print "starting httpd"
+            self.start_httpd()
+
+        if not self.marionette:
+            self.start_marionette()
+            if self.emulator:
+                self.marionette.emulator.wait_for_homescreen(self.marionette)
+            # Retrieve capabilities for later use
+            if not self._capabilities:
+                self.capabilities
+
+        if self.capabilities['device'] != 'desktop':
+            self.textrunnerclass = B2GMarionetteTextTestRunner
+
+        for test in tests:
+            self.add_test(test)
+
         counter = self.repeat
         while counter >=0:
             round = self.repeat - counter
             if round > 0:
                 self.logger.info('\nREPEAT %d\n-------' % round)
-            if self.shuffle:
-                random.shuffle(tests)
-            for test in tests:
-                self.run_test(test)
+            self.run_test_sets()
             counter -= 1
         self.logger.info('\nSUMMARY\n-------')
         self.logger.info('passed: %d' % self.passed)
@@ -758,15 +812,17 @@ class BaseMarionetteTestRunner(object):
         for run_tests in self.mixin_run_tests:
             run_tests(tests)
 
-    def run_test(self, test, expected='pass'):
-        if not self.httpd:
-            print "starting httpd"
-            self.start_httpd()
+    def add_test(self, test, expected='pass', oop=None):
+        filepath = os.path.abspath(test)
 
-        if not self.marionette:
-            self.start_marionette()
-            if self.emulator:
-                self.marionette.emulator.wait_for_homescreen(self.marionette)
+        if os.path.isdir(filepath):
+            for root, dirs, files in os.walk(filepath):
+                for filename in files:
+                    if ((filename.startswith('test_') or filename.startswith('browser_')) and
+                        (filename.endswith('.py') or filename.endswith('.js'))):
+                        filepath = os.path.join(root, filename)
+                        self.add_test(filepath)
+            return
 
         testargs = {}
         if self.type is not None:
@@ -778,66 +834,101 @@ class BaseMarionetteTestRunner(object):
                     testargs.update({ atype[1:]: 'false' })
                 else:
                     testargs.update({ atype: 'true' })
-        oop = testargs.get('oop', False)
-        if isinstance(oop, basestring):
-            oop = False if oop == 'false' else 'true'
 
-        filepath = os.path.abspath(test)
+        # testarg_oop = either None, 'true' or 'false'.
+        testarg_oop = testargs.get('oop')
 
-        if os.path.isdir(filepath):
-            for root, dirs, files in os.walk(filepath):
-                if self.shuffle:
-                    random.shuffle(files)
-                for filename in files:
-                    if ((filename.startswith('test_') or filename.startswith('browser_')) and
-                        (filename.endswith('.py') or filename.endswith('.js'))):
-                        filepath = os.path.join(root, filename)
-                        self.run_test(filepath)
-                        if self.marionette.check_for_crash():
-                            return
-            return
-
-        mod_name,file_ext = os.path.splitext(os.path.split(filepath)[-1])
-
-        testloader = unittest.TestLoader()
-        suite = unittest.TestSuite()
+        file_ext = os.path.splitext(os.path.split(filepath)[-1])[1]
 
         if file_ext == '.ini':
             manifest = TestManifest()
             manifest.read(filepath)
 
-            all_tests = manifest.active_tests(exists=False, disabled=False)
             manifest_tests = manifest.active_tests(exists=False,
-                                                   disabled=False,
+                                                   disabled=True,
                                                    device=self.device,
                                                    app=self.appName,
                                                    **mozinfo.info)
-            skip_tests = list(set([x['path'] for x in all_tests]) -
-                              set([x['path'] for x in manifest_tests]))
-            for skipped in skip_tests:
-                self.logger.info('TEST-SKIP | %s | device=%s, app=%s' %
-                                 (os.path.basename(skipped),
-                                  self.device,
-                                  self.appName))
+            unfiltered_tests = []
+            for test in manifest_tests:
+                if test.get('disabled'):
+                    self.manifest_skipped_tests.append(test)
+                else:
+                    unfiltered_tests.append(test)
+
+            # Don't filter tests with "oop" flag because manifest parser can't
+            # handle it well.
+            if testarg_oop is not None:
+                del testargs['oop']
+
+            target_tests = manifest.get(tests=unfiltered_tests, **testargs)
+            for test in unfiltered_tests:
+                if test['path'] not in [x['path'] for x in target_tests]:
+                    test.setdefault('disabled', 'filtered by type (%s)' % self.type)
+                    self.manifest_skipped_tests.append(test)
+
+            for test in self.manifest_skipped_tests:
+                self.logger.info('TEST-SKIP | %s | %s' % (
+                    os.path.basename(test['path']),
+                    test['disabled']))
                 self.todo += 1
 
-            target_tests = manifest.get(tests=manifest_tests, **testargs)
-            if self.shuffle:
-                random.shuffle(target_tests)
             for i in target_tests:
                 if not os.path.exists(i["path"]):
                     raise IOError("test file: %s does not exist" % i["path"])
-                self.run_test(i["path"], i["expected"])
-                if self.marionette.check_for_crash():
-                    return
+
+                # manifest_oop is either 'false', 'true' or 'both'.  Anything
+                # else implies 'false'.
+                manifest_oop = i.get('oop', 'false')
+
+                # We only add an oop test when following conditions are met:
+                # 1) It's written by javascript because we have only
+                #    MarionetteJSTestCase that supports oop mode.
+                # 2) we're running with "--type=+oop" or no "--type=-oop", which
+                #    follows testarg_oop is either None or 'true' and must not
+                #    be 'false'.
+                # 3) When no "--type=[+-]oop" is applied, all active tests are
+                #    included in target_tests, so we must filter out those
+                #    really capable of running in oop mode. Besides, oop tests
+                #    must be explicitly specified for backward compatibility. So
+                #    test manifest_oop equals to either 'both' or 'true'.
+                file_ext = os.path.splitext(os.path.split(i['path'])[-1])[-1]
+                if (file_ext == '.js' and
+                    testarg_oop != 'false' and
+                    (manifest_oop == 'both' or manifest_oop == 'true')):
+                    self.add_test(i["path"], i["expected"], True)
+
+                # We only add an in-process test when following conditions are
+                # met:
+                # 1) we're running with "--type=-oop" or no "--type=+oop", which
+                #    follows testarg_oop is either None or 'false' and must not
+                #    be 'true'.
+                # 2) When no "--type=[+-]oop" is applied, all active tests are
+                #    included in target_tests, so we must filter out those
+                #    really capable of running in in-process mode.
+                if (testarg_oop != 'true' and
+                    (manifest_oop == 'both' or manifest_oop != 'true')):
+                    self.add_test(i["path"], i["expected"], False)
             return
 
-            self.logger.info('TEST-START %s' % os.path.basename(test))
+        if oop is None:
+            # This test is added by directory enumeration or directly specified
+            # in argument list.  We have no manifest information here so we just
+            # respect the "--type=[+-]oop" argument here.
+            oop = file_ext == '.js' and testarg_oop == 'true'
 
+        self.tests.append({'filepath': filepath, 'expected': expected, 'oop': oop})
+
+    def run_test(self, filepath, expected, oop):
+        self.logger.info('TEST-START %s' % os.path.basename(filepath))
+
+        testloader = unittest.TestLoader()
+        suite = unittest.TestSuite()
         self.test_kwargs['expected'] = expected
         self.test_kwargs['oop'] = oop
+        mod_name = os.path.splitext(os.path.split(filepath)[-1])[0]
         for handler in self.test_handlers:
-            if handler.match(os.path.basename(test)):
+            if handler.match(os.path.basename(filepath)):
                 handler.add_tests_to_suite(mod_name,
                                            filepath,
                                            suite,
@@ -849,7 +940,8 @@ class BaseMarionetteTestRunner(object):
 
         if suite.countTestCases():
             runner = self.textrunnerclass(verbosity=3,
-                                          marionette=self.marionette)
+                                          marionette=self.marionette,
+                                          capabilities=self.capabilities)
             results = runner.run(suite)
             self.results.append(results)
 
@@ -866,6 +958,22 @@ class BaseMarionetteTestRunner(object):
             if hasattr(results, 'expectedFailures'):
                 self.passed += len(results.expectedFailures)
 
+    def run_test_set(self, tests):
+        if self.shuffle:
+            random.shuffle(tests)
+
+        for test in tests:
+            self.run_test(test['filepath'], test['expected'], test['oop'])
+            if self.marionette.check_for_crash():
+                break
+
+    def run_test_sets(self):
+        oop_tests = [x for x in self.tests if x.get('oop')]
+        self.run_test_set(oop_tests)
+
+        in_process_tests = [x for x in self.tests if not x.get('oop')]
+        self.run_test_set(in_process_tests)
+
     def cleanup(self):
         if self.httpd:
             self.httpd.stop()
@@ -874,17 +982,32 @@ class BaseMarionetteTestRunner(object):
 
     def generate_xml(self, results_list):
 
-        def _extract_xml(test, result='passed'):
+        def _extract_xml_from_result(test_result, result='passed'):
+            _extract_xml(
+                test_name=unicode(test_result.name).split()[0],
+                test_class=test_result.test_class,
+                duration=test_result.duration,
+                result=result,
+                output='\n'.join(test_result.output))
+
+        def _extract_xml_from_skipped_manifest_test(test):
+            _extract_xml(
+                test_name=test['name'],
+                result='skipped',
+                output=test['disabled'])
+
+        def _extract_xml(test_name, test_class='', duration=0,
+                         result='passed', output=''):
             testcase = doc.createElement('testcase')
-            testcase.setAttribute('classname', test.test_class)
-            testcase.setAttribute('name', unicode(test.name).split()[0])
-            testcase.setAttribute('time', str(test.duration))
+            testcase.setAttribute('classname', test_class)
+            testcase.setAttribute('name', test_name)
+            testcase.setAttribute('time', str(duration))
             testsuite.appendChild(testcase)
 
             if result in ['failure', 'error', 'skipped']:
                 f = doc.createElement(result)
                 f.setAttribute('message', 'test %s' % result)
-                f.appendChild(doc.createTextNode(test.reason))
+                f.appendChild(doc.createTextNode(output))
                 testcase.appendChild(f)
 
         doc = dom.Document()
@@ -905,33 +1028,38 @@ class BaseMarionetteTestRunner(object):
                                                for results in results_list])))
         testsuite.setAttribute('errors', str(sum([len(results.errors)
                                              for results in results_list])))
-        testsuite.setAttribute('skips', str(sum([len(results.skipped) +
-                                                     len(results.expectedFailures)
-                                                     for results in results_list])))
+        testsuite.setAttribute(
+            'skips', str(sum([len(results.skipped) +
+                         len(results.expectedFailures)
+                         for results in results_list]) +
+                         len(self.manifest_skipped_tests)))
 
         for results in results_list:
 
             for result in results.errors:
-                _extract_xml(result, result='error')
+                _extract_xml_from_result(result, result='error')
 
             for result in results.failures:
-                _extract_xml(result, result='failure')
+                _extract_xml_from_result(result, result='failure')
 
             if hasattr(results, 'unexpectedSuccesses'):
                 for test in results.unexpectedSuccesses:
                     # unexpectedSuccesses is a list of Testcases only, no tuples
-                    _extract_xml(test, result='failure')
+                    _extract_xml_from_result(test, result='failure')
 
             if hasattr(results, 'skipped'):
                 for result in results.skipped:
-                    _extract_xml(result, result='skipped')
+                    _extract_xml_from_result(result, result='skipped')
 
             if hasattr(results, 'expectedFailures'):
                 for result in results.expectedFailures:
-                    _extract_xml(result, result='skipped')
+                    _extract_xml_from_result(result, result='skipped')
 
             for result in results.tests_passed:
-                _extract_xml(result)
+                _extract_xml_from_result(result)
+
+        for test in self.manifest_skipped_tests:
+            _extract_xml_from_skipped_manifest_test(test)
 
         doc.appendChild(testsuite)
         return doc.toprettyxml(encoding='utf-8')
